﻿using System.Collections.Generic;
using UnityEngine;

public class BackgroundBehaviour : MonoBehaviour
{
    [Header("Component References")]


    [SerializeField]
    private Transform m_LayerAttachmentPoint = null;
    private Transform LayerAttachmentPoint
    {
        get { return m_LayerAttachmentPoint; }
    }

    [SerializeField]
    private LevelBoundsRectBehaviour m_LevelBoundsRect = null;

    private void OnEnable()
    {
        BackgroundBehaviour.LevelBoundsRect = m_LevelBoundsRect;
    }

    private void OnDisable()
    {
        BackgroundBehaviour.LevelBoundsRect = null;
    }

    public static LevelBoundsRectBehaviour LevelBoundsRect { get; private set; }



    [Header("Configuration")]
    
    //[SerializeField]
    //[Tooltip("Width to generate in units")]
    //private float m_Width = 12;
    //private float Width
    //{
    //    get { return m_Width; }
    //    set { m_Width = value; }
    //}


    //[SerializeField]
    //[Tooltip("Height to generate in units")]
    //private float m_Height = 8;
    //private float Height
    //{
    //    get { return m_Height; }
    //    set { m_Height = value; }
    //}

    [SerializeField]
    private int m_StartSortingOrder = -1;
    public int StartSortingOrder
    {
        get { return m_StartSortingOrder; }
    }

    [Header("Project References")]

    [SerializeField]
    private Sprite m_SkyTexture;
    private Sprite SkyTexture
    {
        get { return m_SkyTexture; }
        set { m_SkyTexture = value; }
    }


    [SerializeField]
    private List<BackgroundLayerDescriptor> m_LayerDescriptors = null;
    private List<BackgroundLayerDescriptor> LayerDescriptors
    {
        get { return m_LayerDescriptors; }
        set { m_LayerDescriptors = value; }
    }


    [HideInInspector]
    [SerializeField]
    private List<GameObject> m_Layers = null;
    private List<GameObject> Layers
    {
        get { return m_Layers; }
        set { m_Layers = value; }
    }



    public void Clear()
    {
        foreach (var layer in Layers)
        {
            if (Application.isEditor)
                DestroyImmediate(layer.gameObject);
            else
                Destroy(layer.gameObject);
        }
    }

    public void Generate()
    {
        Clear();

        BackgroundBehaviour.LevelBoundsRect = m_LevelBoundsRect;
        
        LayerAttachmentPoint.position = LevelBoundsRect.Center;

        int sortingOrder = StartSortingOrder;

        foreach (var layerDescriptor in LayerDescriptors)
        {
            var width = LevelBoundsRect.Width * 1.25f;


            var layerGo = new GameObject(string.Format("Layer {0}", layerDescriptor.Name));

            var layer = layerGo.AddComponent<BackgroundLayerBehaviour>();

            layer.Distance = layerDescriptor.Distance;
            layer.YRatio = layerDescriptor.YParallaxModifier;


            layerGo.transform.SetParent(LayerAttachmentPoint, false);
            //layerGo.transform.position = LevelBoundsRect.Size;

            layerDescriptor.Generate(layerGo.transform, ref sortingOrder, width );

            Layers.Add(layerGo);
        }

        Debug.Log("sortingOrder is " + sortingOrder);
    }
}

