﻿using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

[Serializable]
public class BackgroundLayerDescriptor : System.Object
{
    [SerializeField]
    private string m_Name = "Layer";
    public string Name
    {
        get { return m_Name; }
    }


    [SerializeField]
    private float m_Distance = 10.0f;
    public float Distance
    {
        get { return m_Distance; }
    }

    
    [SerializeField]
    private List<BackgroundDecorationDescriptor> m_DecorationDescriptors = null;
    private List<BackgroundDecorationDescriptor> DecorationDescriptors
    {
        get { return m_DecorationDescriptors; }
    }


    [SerializeField]
    private BackgroundLandscapeDescriptor m_LandscapeDescriptor = null;
    private BackgroundLandscapeDescriptor LandscapeDescriptor
    {
        get { return m_LandscapeDescriptor; }
    }


    [SerializeField]
    private float m_YParallaxModifier = 0.05f;
    public float YParallaxModifier
    {
        get { return m_YParallaxModifier; }
        set { m_YParallaxModifier = value; }
    }


    //[HideInInspector]
    //[SerializeField]
    //private List<GameObject> m_Decorations = null;
    //private List<GameObject> Decorations
    //{
    //    get { return m_Decorations; }
    //}

    public void Generate(Transform parentTransform, ref int sortingOrder, float width )
    {

        var rand = new System.Random();

        var decorationRenderers = new List<SpriteRenderer>();

        if (DecorationDescriptors.Any())
        {
            foreach (var decorationDescriptor in DecorationDescriptors)
            {
                // Move this into BackgroundDecorationDescriptor
                var decorationAttachmentPoint = new GameObject(decorationDescriptor.Name).transform;
                decorationAttachmentPoint.SetParent(parentTransform, false);

                var currentX = -width / 2.0f;

                while (currentX < width / 2.0f)
                {
                    var newGo = new GameObject(String.Format("{0}_{1}", decorationDescriptor.Name, decorationRenderers.Count));
                    
                    newGo.transform.SetParent(decorationAttachmentPoint);

                    currentX += UnityEngine.Random.Range(-decorationDescriptor.SpacingDeviation, decorationDescriptor.SpacingDeviation);

                    var y = decorationDescriptor.VerticalPosition;
                    y += UnityEngine.Random.Range(-decorationDescriptor.VerticalDeviation, decorationDescriptor.VerticalDeviation);

                    newGo.transform.SetParent(decorationAttachmentPoint, false);
                    newGo.transform.localPosition = new Vector2(currentX, y);

                    var spriteRenderer = newGo.AddComponent<SpriteRenderer>();
                    spriteRenderer.sprite = decorationDescriptor.Sprite;
                    spriteRenderer.flipX = UnityEngine.Random.Range(0, 2) == 0;
                    //spriteRenderer.sortingOrder = sortingOrder--;

                    decorationRenderers.Add(spriteRenderer);

                    currentX += decorationDescriptor.Spacing;
                }

            }

            SetDecorationSortOrder(decorationRenderers, ref sortingOrder);
        }
        
        LandscapeDescriptor.Generate(parentTransform, ref sortingOrder, width );
    }

    private void SetDecorationSortOrder(IList<SpriteRenderer> decorations, ref int sortingOrder)
    {
        //int sortIndex = decorations.Max(d => d.sortingOrder);
        //int max = Decorations.Max(d => d.GetComponent<SpriteRenderer>().sortingOrder);

        foreach (var d in decorations.OrderBy(d => d.transform.position.y))
            d.sortingOrder = sortingOrder--;
    }
}

